@@ -4,7 +4,8 @@ def _format_attr(attr, width):
     s = ''
     a = attr[0]
     len_a = len(a)
-    start = int(width / 2) - int(len_a / 2)
+    shift_left = 1 if (len_a % 2 == 1 and width % 2 == 0 and len_a < width) else 0
+    start = int(width / 2) - int(len_a / 2) - shift_left
     for i in range(0, start):
         s += ' '
     s += a
@@ -26,6 +27,14 @@ def _rstrip_lines(lines):
         newlines.append(l.rstrip())
     return newlines
 
+def _format_value(value, dtype):
+    if len(value) > 1:
+        return value
+    if dtype == 'bool' or dtype == 16:
+        return ['t'] if value[0] == 'True' else ['f']
+    else:
+        return value
+
 def _format_row(row, attrs, width):
     s = ''
     # Count number of lines
@@ -35,7 +44,7 @@ def _format_row(row, attrs, width):
     for i, data in enumerate(row):
         lines = ['']
         if data is not None:
-            lines = _rstrip_lines(str(data).splitlines())
+            lines = _format_value(_rstrip_lines(str(data).splitlines()), attrs[i][1])
         maxlen.append(_maxlen(lines))
         rowlines.append(lines)
         lines_len = len(lines)
@@ -44,17 +53,15 @@ def _format_row(row, attrs, width):
     # Write lines
     for i in range(0, maxlines):
         for j, lines in enumerate(rowlines):
+            lines_i = lines[i] if i < len(lines) else ''
             s += ' ' if j == 0 else '| '
-            if attrs[j][1] == 'NUMBER':
-                start = width[j] - maxlen[j]
+            if attrs[j][1] == 'NUMBER' or attrs[j][1] == 23:
+                start = width[j] - len(lines_i)
             else:
                 start = 0
             for k in range(0, start):
                 s += ' '
-            if i < len(lines):
-                s += lines[i]
-            else:
-                s += ' '
+            s += lines_i
             for k in range(0, width[j] - start - maxlen[j]):
                 s += ' '
             s += ' '
